package com.qmth.sop.business.bean.result;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.sop.common.enums.FlowStatusEnum;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.enums.TFCustomTypeEnum;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: appinfo result
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2023/2/17
 */
public class SopInfoResult implements Serializable {

    @ApiModelProperty(value = "sopid")
    @JsonSerialize(using = ToStringSerializer.class)
    Long id;

    @ApiModelProperty(value = "服务单元id")
    @JsonSerialize(using = ToStringSerializer.class)
    Long serviceId;

    @ApiModelProperty(value = "服务单元名称")
    String serviceName;

    @ApiModelProperty(value = "sop单号")
    String sopNo;

    @ApiModelProperty(value = "crm单号")
    String crmNo;

    @ApiModelProperty(value = "派单时间")
    Long beginTime;

    @ApiModelProperty(value = "客户经理id")
    @JsonSerialize(using = ToStringSerializer.class)
    Long customManagerId;

    @ApiModelProperty(value = "客户类型，OFFICE：教务处，CLOUD_MARK：研究生")
    ProductTypeEnum customManagerType;

    @ApiModelProperty(value = "客户类型str")
    String customManagerTypeStr;

    @ApiModelProperty(value = "客户名称")
    String customName;

    @ApiModelProperty(value = "crm名称")
    String crmName;

    @ApiModelProperty(value = "产品id")
    @JsonSerialize(using = ToStringSerializer.class)
    Long productId;

    @ApiModelProperty(value = "产品名称")
    String productName;

    @ApiModelProperty(value = "考试开始时间")
    Long examStartTime;

    @ApiModelProperty(value = "考试结束时间")
    Long examEndTime;

    @ApiModelProperty(value = "提交人id")
    @JsonSerialize(using = ToStringSerializer.class)
    Long flowCreateId;

    @ApiModelProperty(value = "提交人")
    String flowCreateName;

    @ApiModelProperty(value = "提交时间")
    Long flowCreateTime;

    @ApiModelProperty(value = "更新时间")
    Long flowUpdateTime;

    @ApiModelProperty(value = "流程状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束")
    FlowStatusEnum status;

    @ApiModelProperty(value = "流程状态str")
    String statusStr;

    @ApiModelProperty(value = "流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程")
    TFCustomTypeEnum type;

    @ApiModelProperty(value = "流程类型str")
    String typeStr;

    @ApiModelProperty(value = "流程版本号")
    Integer version;

    @ApiModelProperty(value = "流程部署id")
    String flowDeploymentId;

    @ApiModelProperty(value = "流程id")
    @JsonSerialize(using = ToStringSerializer.class)
    Long flowId;

    @ApiModelProperty(value = "流程节点名称")
    String taskName;

    @ApiModelProperty(value = "流程节点key")
    String taskDefKey;

    @ApiModelProperty(value = "流程节点id")
    @JsonSerialize(using = ToStringSerializer.class)
    Long taskId;

    @ApiModelProperty(value = "当前节点负责人")
    String pendApproveName;

    @ApiModelProperty(value = "动态节点")
    Map<String, Object> dynamicFieldMap;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getServiceId() {
        return serviceId;
    }

    public void setServiceId(Long serviceId) {
        this.serviceId = serviceId;
    }

    public String getServiceName() {
        return serviceName;
    }

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    public String getSopNo() {
        return sopNo;
    }

    public void setSopNo(String sopNo) {
        this.sopNo = sopNo;
    }

    public String getCrmNo() {
        return crmNo;
    }

    public void setCrmNo(String crmNo) {
        this.crmNo = crmNo;
    }

    public Long getBeginTime() {
        return beginTime;
    }

    public void setBeginTime(Long beginTime) {
        this.beginTime = beginTime;
    }

    public Long getCustomManagerId() {
        return customManagerId;
    }

    public void setCustomManagerId(Long customManagerId) {
        this.customManagerId = customManagerId;
    }

    public ProductTypeEnum getCustomManagerType() {
        return customManagerType;
    }

    public void setCustomManagerType(ProductTypeEnum customManagerType) {
        this.customManagerType = customManagerType;
    }

    public String getCustomManagerTypeStr() {
        if (Objects.nonNull(this.customManagerType)) {
            return this.customManagerType.getTitle();
        } else {
            return customManagerTypeStr;
        }
    }

    public void setCustomManagerTypeStr(String customManagerTypeStr) {
        this.customManagerTypeStr = customManagerTypeStr;
    }

    public String getCustomName() {
        return customName;
    }

    public void setCustomName(String customName) {
        this.customName = customName;
    }

    public String getCrmName() {
        return crmName;
    }

    public void setCrmName(String crmName) {
        this.crmName = crmName;
    }

    public Long getProductId() {
        return productId;
    }

    public void setProductId(Long productId) {
        this.productId = productId;
    }

    public String getProductName() {
        return productName;
    }

    public void setProductName(String productName) {
        this.productName = productName;
    }

    public Long getExamStartTime() {
        return examStartTime;
    }

    public void setExamStartTime(Long examStartTime) {
        this.examStartTime = examStartTime;
    }

    public Long getExamEndTime() {
        return examEndTime;
    }

    public void setExamEndTime(Long examEndTime) {
        this.examEndTime = examEndTime;
    }

    public Long getFlowCreateId() {
        return flowCreateId;
    }

    public void setFlowCreateId(Long flowCreateId) {
        this.flowCreateId = flowCreateId;
    }

    public String getFlowCreateName() {
        return flowCreateName;
    }

    public void setFlowCreateName(String flowCreateName) {
        this.flowCreateName = flowCreateName;
    }

    public Long getFlowCreateTime() {
        return flowCreateTime;
    }

    public void setFlowCreateTime(Long flowCreateTime) {
        this.flowCreateTime = flowCreateTime;
    }

    public Long getFlowUpdateTime() {
        return flowUpdateTime;
    }

    public void setFlowUpdateTime(Long flowUpdateTime) {
        this.flowUpdateTime = flowUpdateTime;
    }

    public FlowStatusEnum getStatus() {
        return status;
    }

    public void setStatus(FlowStatusEnum status) {
        this.status = status;
    }

    public String getStatusStr() {
        if (Objects.nonNull(this.status)) {
            return this.status.getTitle();
        } else {
            return statusStr;
        }
    }

    public void setStatusStr(String statusStr) {
        this.statusStr = statusStr;
    }

    public TFCustomTypeEnum getType() {
        return type;
    }

    public void setType(TFCustomTypeEnum type) {
        this.type = type;
    }

    public String getTypeStr() {
        if (Objects.nonNull(this.type)) {
            return this.type.getTitle();
        } else {
            return typeStr;
        }
    }

    public void setTypeStr(String typeStr) {
        this.typeStr = typeStr;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public String getFlowDeploymentId() {
        return flowDeploymentId;
    }

    public void setFlowDeploymentId(String flowDeploymentId) {
        this.flowDeploymentId = flowDeploymentId;
    }

    public Long getFlowId() {
        return flowId;
    }

    public void setFlowId(Long flowId) {
        this.flowId = flowId;
    }

    public String getTaskName() {
        return taskName;
    }

    public void setTaskName(String taskName) {
        this.taskName = taskName;
    }

    public String getTaskDefKey() {
        return taskDefKey;
    }

    public void setTaskDefKey(String taskDefKey) {
        this.taskDefKey = taskDefKey;
    }

    public Long getTaskId() {
        return taskId;
    }

    public void setTaskId(Long taskId) {
        this.taskId = taskId;
    }

    public String getPendApproveName() {
        return pendApproveName;
    }

    public void setPendApproveName(String pendApproveName) {
        this.pendApproveName = pendApproveName;
    }

    public Map<String, Object> getDynamicFieldMap() {
        return dynamicFieldMap;
    }

    public void setDynamicFieldMap(Map<String, Object> dynamicFieldMap) {
        this.dynamicFieldMap = dynamicFieldMap;
    }
}
